import config from './config';
import { db } from './tq_firebase';

const { app } = require('electron').remote;
const log = require('electron-log');
const jetpack = require('fs-jetpack');
const path = require('path');

// Check if there is a newer version of the launcher application available
export async function hasNewLauncherVersion() {
  const version = app.getVersion().split('.')[0];
  const doc = await db.collection('launcherVersion').doc('latest').get();
  const latest = doc.data().version;
  console.log('version:', version);
  console.log('latest:', latest);
  return version !== latest;
}

export function fetchChannels() {
  async function doFetch(resolve, reject) {
    let cachedChannelsList = null;
    try {
      // Load cached list of channels, if available.
      const cacheFile = path.resolve(config.cacheDir, `channels.json`);
      if (jetpack.exists(cacheFile)) {
        const channelsJson = jetpack.read(cacheFile);
        if (channelsJson) {
          cachedChannelsList = JSON.parse(channelsJson);
        }
      }

      // Grab latest channel list from Firebase
      const channelList = [];
      const snapshot = await db.collection('releaseChannels')
        .orderBy('priority')
        .get();
      
      snapshot.forEach(doc => {
        const docData = doc.data();
        docData.id = doc.id;
        channelList.push(docData);
      });

      // Store the most recent result in a file to use as a fallback
      // when offline, IF we successfully actually got something from
      // Firebase
      if (channelList && channelList.length > 0) {
        jetpack.write(cacheFile, JSON.stringify(channelList));
        resolve(channelList);
      } else if(cachedChannelsList) {
        resolve(cachedChannelsList);
      } else {
        resolve([]);
      }
    } catch(e) {
      log.error(`Error fetching channel list`, e);

      if(cachedChannelsList) {
        return resolve(cachedChannelsList);
      }

      reject(e);
    }
  }

  return new Promise((resolve, reject) => {
    doFetch(resolve, reject);
  });
}

export function fetchReleases(channelName) {
  async function doFetch(resolve, reject) {
    let cachedReleaseList = null;
    try {
      const cacheFile = path.resolve(config.cacheDir, `releases-${channelName}.json`);
      if (jetpack.exists(cacheFile)) {
        const releaseJson = jetpack.read(cacheFile);
        if (releaseJson) {
          cachedReleaseList = JSON.parse(releaseJson);
        }
      }

      // Grab the last several releases 
      const releaseList = [];
      const snapshot = await db.collection('releases')
        .where('channelName', '==', channelName)
        .orderBy('createdAt', 'desc')
        .orderBy('name', 'desc')
        .get();
      
      snapshot.forEach(doc => {
        // Massage release data into the format we want
        const docData = {
          id: doc.id,
          createdAt: doc.get('createdAt').toDate(),
          name: doc.get('name'),
          checksums: doc.get('checksums'),
          channelName
        };
        releaseList.push(docData);
      });

      // Store the most recent result in a file to use as a fallback
      // when offline, IF we successfully actually got something from
      // Firebase
      if (releaseList && releaseList.length > 0) {
        jetpack.write(cacheFile, JSON.stringify(releaseList));
        resolve(releaseList);
      } else if (cachedReleaseList) {
        resolve(cachedReleaseList);
      } else {
        resolve([]);
      }
    } catch(e) {
      log.error(`Error fetching release list`, e);

      if (cachedReleaseList) {
        return resolve(cachedReleaseList);
      }

      reject(e);
    }
  }

  return new Promise((resolve, reject) => {
    doFetch(resolve, reject);
  });
}
